local Highlighting =  FlexBar2:NewModule("Highlighting");
Highlighting.Version = 1;
local Aura = AceLibrary("SpecialEvents-Aura-2.0");
local PaintChips = AceLibrary("PaintChips-2.0");

local InnerPriority = {"NoRange", "NoMana", "Unusable"};
local OuterPriority = {"Channeling", "Casting","AutoRepeating", "Buff", "Debuff", "Equipped"};
local RangeGuess;

function Highlighting:OnEnable()
    PaintChips:RegisterColor("Buff", 0 , 255, 0);
    PaintChips:RegisterColor("Debuff", 255 , 0, 0);
    PaintChips:RegisterColor("Equipped", 0 , 0, 255);
    PaintChips:RegisterColor("NoMana", 0 , 0, 150);
    PaintChips:RegisterColor("NoRange", 150 , 0, 0);
    PaintChips:RegisterColor("Unusable", 100 , 100, 100);
    PaintChips:RegisterColor("Casting", 255, 255, 255);
    PaintChips:RegisterColor("Channeling", 148, 0, 211);
    PaintChips:RegisterColor("AutoCasting", 238, 118, 0);
end

Highlighting.ButtonMixin = FlexBar2.OO.Mixin
{
    "SetOuterHighlight",
    "SetInnerHighlight",
    "UpdateOuterHighlight",
    "CheckCasting",
    "CheckChanneling",
    "CheckStartAutoCasting",
    "CheckStopAutoCasting",
    "CheckBuff",
    "CheckDebuff",
    "CheckUsable",
    "CheckEquipped",
    "CheckRange",
};

function Highlighting.ButtonMixin:Activate()
    self.InnerCheck = {};
    self.OuterCheck = {};
    self:RegisterScript("PostClick", "UpdateOuterHighlight");
    self:RegisterEvent("FB2_UPDATE", "CheckRange");
    self:RegisterEvent("UNIT_MANA", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckUsable(); end end);
    self:RegisterEvent("UNIT_RAGE", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckUsable(); end end);
    self:RegisterEvent("UNIT_ENERGY", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckUsable(); end end);
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckEquipped(); end end);
    self:RegisterEvent("UNIT_SPELLCAST_START", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckCasting(); end end);
    self:RegisterEvent("UNIT_SPELLCAST_STOP", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckCasting(); end end);
    self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckChanneling(); end end);
    self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", function(Button, Event, Source, UnitId) if(UnitId == "player") then self:CheckChanneling(); end end);
    self:RegisterEvent("START_AUTOREPEAT_SPELL", "CheckStartAutoCasting");
    self:RegisterEvent("STOP_AUTOREPEAT_SPELL", "CheckStopAutoCasting");
    self:RegisterEvent("PLAYER_ENTER_COMBAT", "CheckStartAutoCasting");
    self:RegisterEvent("PLAYER_LEAVE_COMBAT", "CheckStopAutoCasting");
    self:RegisterEvent("SpecialEvents_PlayerBuffGained", "CheckBuff");
    self:RegisterEvent("SpecialEvents_PlayerBuffLost", "CheckBuff");
    self:RegisterEvent("SpecialEvents_UnitDebuffGained", "CheckDebuff");
    self:RegisterEvent("SpecialEvents_UnitDebuffLost", "CheckDebuff");
    self:RegisterEvent("SpecialEvents_AuraTargetChanged", "CheckDebuff");
    self:RegisterEvent("FB2_ActionChanged", function(self, Event, Dispatcher, Button) FlexBar2:Debug(self, Button); if(Button == self) then self:CheckBuff(); end end);
    self:RegisterEvent("FB2_ActionChanged", function(self, Event, Dispatcher, Button) if(Button == self) then self:CheckDebuff(); end end);
    self:RegisterEvent("FB2_ActionChanged", function(self, Event, Dispatcher, Button) if(Button == self) then self:CheckChanneling(); end end);
    self:RegisterEvent("FB2_ActionChanged", function(self, Event, Dispatcher, Button) if(Button == self) then self:CheckCasting(); end end);
    self:RegisterEvent("FB2_ActionChanged", function(self, Event, Dispatcher, Button) if(Button == self) then self:CheckEquipped(); end end);
    self:RegisterEvent("FB2_ActionChanged", function(self, Event, Dispatcher, Button) if(Button == self) then self:CheckUsable(); end end);
    self:CheckEquipped();
    self:CheckBuff();
    self:CheckDebuff();
    self:CheckUsable();
end

function Highlighting.ButtonMixin:UpdateOuterHighlight()
    if(type(self.OuterCheck[1]) == "string") then
		self.Frame:SetChecked(true);
		local _, r, g, b = PaintChips:GetRGBPercent(self.OuterCheck[1])
		self.Frame:GetCheckedTexture():SetVertexColor(r, g, b);
	else
		self.Frame:SetChecked(false);
	end
end

function Highlighting.ButtonMixin:SetInnerHighlight(CheckType, State)
	if(State) then
		if(not FlexBar2:ValueIsInTable(self.InnerCheck, CheckType)) then
			table.insert(self.InnerCheck, CheckType);
			table.sort(self.InnerCheck, function(a, b)
				return FlexBar2:GetKeyByValue(InnerPriority, a) < FlexBar2:GetKeyByValue(InnerPriority, b);
			end);
		end
	else
		if(FlexBar2:RemoveByValue(self.InnerCheck, CheckType)) then
		end
	end
	if(type(self.InnerCheck[1]) == "string") then
		local _, r, g, b = PaintChips:GetRGBPercent(self.InnerCheck[1])
		self.Frame.Icon:SetVertexColor(r, g, b);
    else
        self.Frame.Icon:SetVertexColor(1, 1, 1);
	end
end

function Highlighting.ButtonMixin:SetOuterHighlight(CheckType, State)
	if(State) then
		if(not FlexBar2:ValueIsInTable(self.OuterCheck, CheckType)) then
			table.insert(self.OuterCheck, CheckType);
			table.sort(self.OuterCheck, function(a, b)
				return FlexBar2:GetKeyByValue(OuterPriority, a) < FlexBar2:GetKeyByValue(OuterPriority, b);
			end);
		end
	else
		if(FlexBar2:RemoveByValue(self.OuterCheck, CheckType)) then
		end
	end
	self:UpdateOuterHighlight();
end

function Highlighting.ButtonMixin:CheckStartAutoCasting()
    local Spell = self:GetModifiedAttribute("spell"); 
    if(Spell) then
        self:SetOuterHighlight("AutoCasting", IsAutoRepeatSpell(Spell) or IsAttackSpell(Spell));
    else
        self:SetOuterHighlight("AutoCasting", false);
    end
end

function Highlighting.ButtonMixin:CheckStopAutoCasting()
    self:SetOuterHighlight("AutoCasting", false);
end

function Highlighting.ButtonMixin:CheckCasting()
    local SpellName, SpellRank = UnitCastingInfo("player");
    if(SpellName) then
        if(SpellRank) then
            SpellName = SpellName .. "(" .. SpellRank .. ")";
        end
        self:SetOuterHighlight("Casting", self:GetModifiedAttribute("spell") == SpellName);
    else
        self:SetOuterHighlight("Casting", false);
    end
end

function Highlighting.ButtonMixin:CheckChanneling()
    local SpellName, SpellRank = UnitCastingInfo("player");
    if(SpellName) then
        if(SpellRank) then
            SpellName = SpellName .. "(" .. SpellRank .. ")";
        end
        self:SetOuterHighlight("Channeling", self:GetModifiedAttribute("spell") == SpellName);
    else
        self:SetOuterHighlight("Channeling", false);
    end
end

function Highlighting.ButtonMixin:CheckBuff()
    local Icon = self:GetIcon();
    if(Icon ~= nil and Aura:UnitHasBuff("player", nil, nil, Icon)) then
        self:SetOuterHighlight("Buff", true);
    else
        self:SetOuterHighlight("Buff", false);
    end
end

function Highlighting.ButtonMixin:CheckDebuff()
    if(Aura:UnitHasDebuff("target", nil, nil, self:GetIcon())) then
        self:SetOuterHighlight("Debuff", true);
    else
        self:SetOuterHighlight("Debuff", false);
    end
end

function Highlighting.ButtonMixin:CheckEquipped()
    local Item = self:GetModifiedAttribute("itemlink");
    if(Item and IsEquippedItem(Item)) then
        self:SetOuterHighlight("Equipped", true);
    else
        self:SetOuterHighlight("Equipped", false);
    end
end

function Highlighting.ButtonMixin:CheckRange()
    local ButtonType = self:GetModifiedAttribute("type");
    local IsInRange;
    if(ButtonType == "spell") then
        local Spell = self:GetModifiedAttribute("spell");
        IsInRange = IsSpellInRange(Spell, "target");
    elseif(ButtonType == "item") then
        local Item = self:GetModifiedAttribute("itemlink");
        local IsInRange = IsItemInRange(Item, "target"); 
    else
        self:SetInnerHighlight("NoRange", false);
        return;
    end
    self:SetInnerHighlight("NoRange", IsInRange == 0);
end

function Highlighting.ButtonMixin:CheckUsable()
    local ButtonType = self:GetModifiedAttribute("type");
    local Usable, NoMana;
    if(ButtonType == "spell") then
        local Spell = self:GetModifiedAttribute("spell");
        Usable, NoMana = IsUsableSpell(Spell); 
    elseif(ButtonType == "item") then
        local Item = self:GetModifiedAttribute("itemlink");
        Usable, NoMana = IsUsableItem(Item);
    else
        self:SetInnerHighlight("NoMana", false);
        self:SetInnerHighlight("Unusable", false);
        return;
    end
    if(Usable) then
            self:SetInnerHighlight("Unusable", false);
            self:SetInnerHighlight("NoMana", false);
    else
        self:SetInnerHighlight("Unusable", true);
        if(NoMana) then
            self:SetInnerHighlight("NoMana", true);
        else
            self:SetInnerHighlight("NoMana", false);
        end
    end
end
